#include "move.h"

// also used in main.c, but header included, OK ;
// tells which kind of piece is on a square (0 if nothing)
int whichPiece(BoardState* bs, Bitboard square, int color) {
    if (square & bs->pawns[color]) return 1;
    if (square & bs->rooks[color]) return 2;
    if (square & bs->knights[color]) return 3;
    if (square & bs->bishops[color]) return 4;
    if (square & bs->queens[color]) return 5;
    if (square & bs->kings[color]) return 6;
    return 0;
}


// flags updated in main parser ; here we just do move on board
//..AND on each pieces' Bitboard concerned :
void play(BoardState* bs, Move* mv) {
	
    // do move on full board :
    bs->fullBoard = (bs->fullBoard ^ mv->initSquare) | mv->destSquare;

    // do move on the right type of pieces too :
    int captureFlag = mv->flags & 0x07;
	int promoteFlag = mv->flags & 0x38;
    int color = bs->color;
    int pieceType = whichPiece(bs, mv->initSquare, color);
    switch (pieceType) {

    case 1:
        bs->pawns[color] = (bs->pawns[color] ^ mv->initSquare) | mv->destSquare;
        if (promoteFlag == 0x28) {
            // en passant
            if (color) bs->pawns[0] ^= (mv->destSquare << 8);
            else bs->pawns[1] ^= (mv->destSquare >> 8);
        }
        else if (promoteFlag) {
            // promotion
            bs->pawns[color] ^= mv->destSquare;
            switch (promoteFlag) {
            case 0x08:
                //promote rook :
                bs->rooks[color] |= mv->destSquare;
                break;
            case 0x10:
                //promote knight:
                bs->knights[color] |= mv->destSquare;
                break;
            case 0x18:
                //promote bishop:
                bs->bishops[color] |= mv->destSquare;
                break;
            case 0x20:
                //promote queen:
                bs->queens[color] |= mv->destSquare;
            }
        }
        break;

	// moving pieces rooks, knights, bishops and queens are very easy to handle :
	// no special moves..
    case 2:
        bs->rooks[color] = (bs->rooks[color] ^ mv->initSquare) | mv->destSquare;
        break;

    case 3:
        bs->knights[color] = (bs->knights[color] ^ mv->initSquare) | mv->destSquare;
        break;

    case 4:
        bs->bishops[color] = (bs->bishops[color] ^ mv->initSquare) | mv->destSquare;
        break;

    case 5:
        bs->queens[color] = (bs->queens[color] ^ mv->initSquare) | mv->destSquare;
        break;

	// take care of castle for king :
    case 6:
        bs->kings[color] = mv->destSquare;
        if (captureFlag == 6) {
            //small castle
			Bitboard initRook = mv->initSquare << 3;
			Bitboard destRook = mv->initSquare << 1;
			bs->fullBoard = (bs->fullBoard ^ initRook) | destRook;
            bs->rooks[color] = (bs->rooks[color] ^ initRook) | destRook;
        }
        else if (captureFlag == 7) {
            //big castle
			Bitboard initRook = mv->initSquare >> 4;
			Bitboard destRook = mv->initSquare >> 1;
			bs->fullBoard = (bs->fullBoard ^ initRook) | destRook;
            bs->rooks[color] = (bs->rooks[color] ^ initRook) | destRook;
        }
        break;
    }

	// now check for capture : make the enemy piece disappear
    if (captureFlag && captureFlag <= 5) {
        switch (captureFlag) {
        case 1:
            //pawn capture
            bs->pawns[1-color] ^= mv->destSquare;
            break;
        case 2:
            //rook :
            bs->rooks[1-color] ^= mv->destSquare;
            break;
        case 3:
            //knight :
            bs->knights[1-color] ^= mv->destSquare;
            break;
        case 4:
            //bishop :
            bs->bishops[1-color] ^= mv->destSquare;
            break;
        case 5:
            //queen :
            bs->queens[1-color] ^= mv->destSquare;
            break;
        }
    }

    // finally change board color
    bs->color = 1 - color;
}

// undo move : same as function above, but reversed.
void takeback(BoardState* bs, Move* mv) {
	
    // undo move on full board :
    bs->fullBoard = (bs->fullBoard ^ mv->destSquare) | mv->initSquare;

    // change color to ours :
    bs->color = 1 - bs->color;
    int color = bs->color;

    // undo move on the right type of piece :
    int captureFlag = mv->flags & 0x07;
	int promoteFlag = mv->flags & 0x38;
    int pieceType = whichPiece(bs, mv->destSquare, color);

    if (pieceType==1 || mv->flags >= 0x48) {
		if ( ! promoteFlag )
			bs->pawns[color] = (bs->pawns[color] ^ mv->destSquare) | mv->initSquare;
        else if (promoteFlag == 0x28) {
			bs->pawns[color] = (bs->pawns[color] ^ mv->destSquare) | mv->initSquare;
            // en passant : we give back the taken pawn
            if (color) {
				bs->pawns[0] |= (mv->destSquare << 8);
				bs->fullBoard |= (mv->destSquare << 8);
			}
            else {
				bs->pawns[1] |= (mv->destSquare >> 8);
				bs->fullBoard |= (mv->destSquare >> 8);
			}
        }
        else {
			bs->pawns[color] |= mv->initSquare;
            // promotion : we delete the promote piece
            switch (promoteFlag) {
            case 0x08:
                //promote rook :
                bs->rooks[color] ^= mv->destSquare;
                break;
            case 0x10:
                //promote knight:
                bs->knights[color] ^= mv->destSquare;
                break;
            case 0x18:
                //promote bishop:
                bs->bishops[color] ^= mv->destSquare;
                break;
            case 0x20:
                //promote queen:
                bs->queens[color] ^= mv->destSquare;
            }
        }
    }

	else {
		switch (pieceType) {
		// easy, we just "move the piece backward" :
		case 2:
		    bs->rooks[color] = (bs->rooks[color] ^ mv->destSquare) | mv->initSquare;
		    break;

		case 3:
		    bs->knights[color] = (bs->knights[color] ^ mv->destSquare) | mv->initSquare;
		    break;

		case 4:
		    bs->bishops[color] = (bs->bishops[color] ^ mv->destSquare) | mv->initSquare;
		    break;

		case 5:
		    bs->queens[color] = (bs->queens[color] ^ mv->destSquare) | mv->initSquare;
		    break;

		case 6:
		    bs->kings[color] = mv->initSquare;
		    // undo castle move if needed :
		    if (captureFlag == 6) {
		        //small castle
				Bitboard initRook = mv->initSquare << 3;
				Bitboard destRook = mv->initSquare << 1;
				bs->fullBoard = (bs->fullBoard ^ destRook) | initRook;
        	    bs->rooks[color] = (bs->rooks[color] ^ destRook) | initRook;
		    }
		    else if (captureFlag == 7) {
		        //big castle
				Bitboard initRook = mv->initSquare >> 4;
				Bitboard destRook = mv->initSquare >> 1;
				bs->fullBoard = (bs->fullBoard ^ destRook) | initRook;
        	    bs->rooks[color] = (bs->rooks[color] ^ destRook) | initRook;
		    }
		    break;
    	}
	}

    // and finally give back the captured piece :
    if (captureFlag && captureFlag <= 5) {
		bs->fullBoard |= mv->destSquare;
        switch (captureFlag) {
        case 1:
            //pawn capture
            bs->pawns[1-color] |= mv->destSquare;
            break;
        case 2:
            //rook :
            bs->rooks[1-color] |= mv->destSquare;
            break;
        case 3:
            //knight :
            bs->knights[1-color] |= mv->destSquare;
            break;
        case 4:
            //bishop :
            bs->bishops[1-color] |= mv->destSquare;
            break;
        case 5:
            //queen :
            bs->queens[1-color] |= mv->destSquare;
            break;
        }
    }
}

