#include "stack_sm.h"

// a function to [deep] copy a StateAndMove (BoardState + Move) object ;
// means allocate a full new structure. useful for SM' stacking in main parser
StateAndMove* safe_copySM(StateAndMove* sm) {
    StateAndMove* new_sm = safe_malloc (sizeof(StateAndMove));

    // allocate BoardState:
    BoardState* new_bs = safe_malloc (sizeof(BoardState));
    new_bs->pawns = safe_malloc (2*sizeof(Bitboard)); //[0] : black ; [1]: white
    new_bs->rooks = safe_malloc(2*sizeof(Bitboard));
    new_bs->knights = safe_malloc(2*sizeof(Bitboard));
    new_bs->bishops = safe_malloc(2*sizeof(Bitboard));
    new_bs->queens = safe_malloc(2*sizeof(Bitboard));
    new_bs->kings = safe_malloc(2*sizeof(Bitboard));

    //fill BoardState:
    BoardState* bs = sm->bs;
    new_bs->fullBoard = bs->fullBoard;;
    new_bs->pawns[1] = bs->pawns[1];
    new_bs->pawns[0] = bs->pawns[0];
    new_bs->rooks[1] = bs->rooks[1];
    new_bs->rooks[0] = bs->rooks[0];
    new_bs->knights[1] = bs->knights[1];
    new_bs->knights[0] = bs->knights[0];
    new_bs->bishops[1] = bs->bishops[1];
    new_bs->bishops[0] = bs->bishops[0];
    new_bs->queens[1] = bs->queens[1];
    new_bs->queens[0] = bs->queens[0];
    new_bs->kings[1] = bs->kings[1];
    new_bs->kings[0] = bs->kings[0];
    new_bs->color = bs->color; // 1 means "white to move"

    //allocate and fill Move:
    Move* new_mv = safe_malloc (sizeof(Move));
    Move* mv = sm->mv;
    new_mv->initSquare = mv->initSquare;
    new_mv->destSquare = mv->destSquare;
    new_mv->flags = mv->flags;
    new_mv->vdepth = mv->vdepth;

    new_sm->bs = new_bs;
    new_sm->mv = new_mv;
    return new_sm;
}

//free all memory allocated for a SM' structure :
void free_contentsSM(StateAndMove* sm) {
    free(sm->bs->pawns);
    free(sm->bs->rooks);
    free(sm->bs->knights);
    free(sm->bs->bishops);
    free(sm->bs->queens);
    free(sm->bs->kings);
    free(sm->bs);
    free(sm->mv);
}


/*****************************
 * STACK operations
 */

//..on StateAndMove's :
void pushSM(StackSM** head, StateAndMove* sm) {
    StackSM* node = safe_malloc (sizeof(StackSM)); /* create a new node */
    /* initialize node */
    node->sm = sm;
    node->next = *head; /* insert new head if any */
    *head = node;
    return;
}

StateAndMove* popSM(StackSM** head) {
    /* pop a node */
    StackSM* top = *head;
    if (top == NULL) return NULL;
    StateAndMove* sm = top->sm;
    *head = top->next;
    free(top);
    return sm;
}

//..on int's (for parsing continuation moves) :
void pushInt(StackInt** head, int* cont) {
    StackInt* node = safe_malloc (sizeof(StackInt)); /* create a new node */
    /* initialize node */
    node->cont = cont;
    node->next = *head; /* insert new head if any */
    *head = node;
    return;
}

int* popInt(StackInt** head) {
    /* pop a node */
    StackInt* top = *head;
    if (top == NULL) return NULL;
    int* cont = top->cont;
    *head = top->next;
    free(top);
    return cont;
}

