#include "test_check.h"

// << move DOWN on visual board, >> move UP on visual board.


/** function to tell if a slider move results in a check, and so is illegal
 * this is actually quite light : we only have to check rooks, bishops and queens,
 * because situation happens after OUR piece move ;
 * so our king cannot be checked by opponent pawns or knights.
 */
int checkCheck(BoardState* bs, Bitboard initSquare, Bitboard destSquare) {

    int color = bs->color;
    int sizeLeftShift=0, sizeRightShift=0;
    Bitboard bb_left = 0, bb_right = 0, attakers = 0;

    // ..first convert king Bitboard position and our piece initial square positions
    // into bidimensional (file, rank) coordinates :
    int position_king = ffsll(bs->kings[color]) - 1;
    int file_king = position_king % 8;
    int rank_king = 7 - position_king / 8;
    int position_is = ffsll(initSquare) - 1;
    int file_is = position_is % 8;
    int rank_is = 7 - position_is / 8;

	/** Method
	 *    1) check if our piece was align with our king (otherwise move is always legal)
	 *    2) if yes, find the right type of slider, and perform a basic search
	 *       in every direction, similar to initSquare.c
	 */

    // 1) : try vertical align
    if (file_king == file_is) {

		int position_ds = ffsll(destSquare) - 1;
    	int file_ds = position_ds % 8;

		if (file_ds != file_is) {
        	// check vertical sliders (rooks queens)
        	attakers = (bs->rooks[1-color] | bs->queens[1-color]) & files[file_is];
        	sizeLeftShift = 8;
        	bb_left = ranks[0];
        	sizeRightShift = 8;
        	bb_right = ranks[7];
		}
    }

	// 1) : try horizontal align
    else if (rank_king == rank_is) {

		int position_ds = ffsll(destSquare) - 1;
    	int rank_ds = 7 - position_ds / 8;

		if (rank_ds != rank_is) {
		    // check horizontal sliders (rooks queens)
		    attakers = (bs->rooks[1-color] | bs->queens[1-color]) & ranks[rank_is];
		    sizeLeftShift = 1;
		    bb_left = files[7];
		    sizeRightShift = 1;
		    bb_right = files[0];
		}
    }

	// 1) : try main diagonal align
    else if ( (file_king - rank_king) == (file_is - rank_is) ) {

		int position_ds = ffsll(destSquare) - 1;
    	int file_ds = position_ds % 8;
    	int rank_ds = 7 - position_ds / 8;

		if ( (file_ds - rank_ds) != (file_is - rank_is) ) {

		    // check diagonal sliders (bishops queens) on main diagonal
		    attakers = bs->bishops[1-color] | bs->queens[1-color];
		    sizeLeftShift = 7;
		    sizeRightShift = 7;

		    // find mask : similar way as in init_square.c ;
		    Bitboard diagMask = diagonals[0];
		    if (file_king < rank_king) {
		        // upper part
		        diagMask >>= 8 * (rank_king - file_king);
		        bb_left = files[0];
		        bb_right = ranks[7];
		    }
		    else if (file_king > rank_king) {
		        // lower part of the board
		        diagMask <<= 8 * (file_king - rank_king);
		        bb_left = ranks[0];
		        bb_right = files[7];
		    }
		}
    }

	// 1) : try other diagonal align
    else if ( (file_king + rank_king - 7) == (file_is + rank_is - 7) ) {

		int position_ds = ffsll(destSquare) - 1;
    	int file_ds = position_ds % 8;
    	int rank_ds = 7 - position_ds / 8;

		if ( (file_ds + rank_ds - 7) == (file_is + rank_is - 7) ) {

		    // check diagonal sliders (bishops queens) on other diagonal
		    attakers = bs->bishops[1-color] | bs->queens[1-color];
		    sizeLeftShift = 9;
		    sizeRightShift = 9;

		    Bitboard diagMask = diagonals[1];
		    int mRank_king = 7 - rank_king;
		    if (file_king < mRank_king) {
		        // lower part
		        diagMask <<= 8 * (mRank_king - file_king);
		        bb_left = ranks[0];
		        bb_right = files[0];
		    }
		    else if (file_king > mRank_king) {
		        // upper part
		        diagMask >>= 8 * (file_king - mRank_king);
		        bb_left = files[7];
		        bb_right = ranks[7];
		    }
		}
	}

    if ( ! attakers ) {
        // of course if "bad" configuration or no potential attakers.. all is OK
        return 0;
    }

    // (virtually) do move on full board, and make the king disappear :
    Bitboard newFullBoard = ((bs->fullBoard ^ initSquare) | destSquare) ^ bs->kings[color];

	// 2) : we know the direction, so only two ways to check
    Bitboard moving_kp = bs->kings[color];
    while ( moving_kp && ! (moving_kp & bb_left) && ! (moving_kp & newFullBoard) )
        moving_kp <<= sizeLeftShift;
    if (moving_kp & attakers) return 1;

    // check other way:
    moving_kp = bs->kings[color];
    while ( moving_kp && ! (moving_kp & bb_right) && ! (moving_kp & newFullBoard) )
        moving_kp >>= sizeRightShift;

    if (moving_kp & attakers) return 1;
    return 0; // all is safe !
}
