/*************************************
 * UTILS GENERAL FUNCTIONS :
 ************************************/

// function called on click on a specific game container
/* set the current game number */
function SetGame(gamenum) {
	curgame = gamenum;
}

// return the index of upper left corner of board passed in parameter ;
// first call is costly, but further ones are immediate
function GetImagesIndex() {

	if (imginds[curgame] >= 0) return imginds[curgame]; // already processed

	// search has to be done :
	var tabim = doc.images;
	// we know that at least (64 +1) * curgame images have been printed above,
	// so we can safely begin search at index == 65 * curgame
	var firstInd = 65 * curgame;
	for (var i = firstInd; i < tabim.length; i++) {
		if (tabim[i].name == "brd"+curgame) {

    		// use this index as reference :
			imginds[curgame] = i;
			return i;
		}
	}
}

// return URL according to piece code on internal board
// byBoard == true if switch on board value (for drawing, not for takeback)
function GetPieceURL(i, byBoard) {
	switch ( ( byBoard ? boards[curgame][i] : i ) ) {
	case -6: return bk;
	case -5: return bq;
	case -4: return bb;
	case -3: return bn;
	case -2: return br;
	case -1: return bp;
	case 0: return empty;
	case 1: return wp;
	case 2: return wr;
	case 3: return wn;
	case 4: return wb;
	case 5: return wq;
	case 6: return wk;
	}
	return "";
}

// reset internal board for current game (almost everything is global var.)
function ResetBoard() {

	// black side :
	boards[curgame][0] = -2;
	boards[curgame][1] = -3;
	boards[curgame][2] = -4;
	boards[curgame][3] = -5;
	boards[curgame][4] = -6;
	boards[curgame][5] = -4;
	boards[curgame][6] = -3;
	boards[curgame][7] = -2;
	for (var i=8; i<16; i++) boards[curgame][i] = -1;

	// 32 empty squares :	
	for (var i=16; i<48; i++) boards[curgame][i] = 0;

	// white side :
	for (var i=48; i<56; i++) boards[curgame][i] = 1;
	boards[curgame][56] = 2;
	boards[curgame][57] = 3;
	boards[curgame][58] = 4;
	boards[curgame][59] = 5;
	boards[curgame][60] = 6;
	boards[curgame][61] = 4;
	boards[curgame][62] = 3;
	boards[curgame][63] = 2;
}

// simple move, without taking flags into account
/* move on internal board */
function BasicMoveInternal(initSquare, destSquare) {
	boards[curgame][destSquare] = boards[curgame][initSquare];
	if (initSquare != destSquare) boards[curgame][initSquare] = 0;
}

// simple move, without taking flags into account
/* move on visual (HTML) board */
function BasicMoveHTML(initSquare, destSquare) {
	var indImg = GetImagesIndex(); // index in the chessboard table
	// orientation has to be watched :
	var destsq = (flipped[curgame] ? 63-destSquare : destSquare);
	var initsq = (flipped[curgame] ? 63-initSquare : initSquare);
	doc.images[indImg + destsq].src = doc.images[indImg + initsq].src;
	if (initSquare != destSquare) doc.images[indImg + initsq].src = empty;
}

// draw a piece or empty square at specific location
// (for back captures, promotions, en passant)
function SetOnHTML(square, url) {
	var sq = (flipped[curgame] ? 63-square : square);
	var indImg = GetImagesIndex(); // index in the chessboard table
	doc.images[indImg + sq].src = url;
}

/* draw the board from internal chessboard : (after e.g. backFive, goToEnd..)
 * NOTE : internal chessboard is never flipped
 * ==> boards[curgame][i] == i-th from upper left on game num. curgame
 * code = 6 king, 5 queen, 4 bishop, 3 knight, 2 rook, 1 pawn, 0 empty,
 * and opposite for black side.
 */
function DrawBoard() {
	var tabim = doc.images;
	var index = GetImagesIndex();

	// go through the array, assigning right url's :
	for (var i=0; i<64; i++) {
		tabim[index+i].src = ( flipped[curgame] ? GetPieceURL(63-i, true) : GetPieceURL(i, true) );
	}
}

/* Function for highlighting moves just after played ;
 * -1 indicates there's nothing to turn on/off
 * otherwise : have to print nicely (especially with move numbers)
 */
function Highlight(turnoff, turnon) {

	var nullOff = 0;
	for (var i=0; i<turnoff; i+=4) {
		if (games[curgame][i] == games[curgame][i+1]) nullOff+=4;
	}
	turnoff -= nullOff;
	var nullOn = 0;
	for (var i=0; i<turnon; i+=4) {
		if (games[curgame][i] == games[curgame][i+1]) nullOn+=4;
	}
	turnon -= nullOn;
	var ctrPan = doc.getElementById("mp"+curgame);

	if (turnoff >= 0) {
		// turn something off : easy
		var moveNum = turnoff / 4;
		ctrPan.innerHTML = "initial position"; // transitional..
		var curMoveTxt = doc.getElementById("m"+curgame+"_"+moveNum);
		curMoveTxt.style.color = movescol;
		curMoveTxt.style.fontWeight = "normal";
	}

	if (turnon >= 0) {
		// turn something on :
		var moveNum = turnon / 4;
		var curMove = doc.getElementById("m"+curgame+"_"+moveNum);
		if (curMove.innerHTML[0] >= '1' && curMove.innerHTML[0] <= '9') {
			// a move with number : print as it is
			ctrPan.innerHTML = curMove.innerHTML;
		}
		else {
			// a bit harder : no number ; this means black move, the number is on previous move
			strWithNum = doc.getElementById("m"+curgame+"_"+(moveNum-1)).innerHTML;
			var i=0;
			while (strWithNum[i] >= '0' && strWithNum[i] <= '9') i++;
			strNum = strWithNum.substr(0, i);
			ctrPan.innerHTML = strNum+"... "+curMove.innerHTML;
		}
		// finally highlight move :
		var curMoveTxt = doc.getElementById("m"+curgame+"_"+moveNum);
		curMoveTxt.style.color = lmovecol;
		curMoveTxt.style.fontWeight = "bold";
	}
}


/**************************
 * Go to a specific move :
 *************************/

//only on mouse click : go "directly" to some position [TODO: can be more efficient]
function GotoMove(moveNum) {

	// the method is simple : reset, then replay all
	ResetBoard();
	
	// find the REAL index of move asked (null moves can exist..)
	var realIndex = 0;
	for (ctrl=0; ctrl < (4*moveNum); ctrl+=4) {
		realIndex+=4;
		if (games[curgame].charCodeAt(ctrl+4) == games[curgame].charCodeAt(ctrl+5))
			realIndex+=4;
	}

	// take care of printing/highlighting :
	Highlight(curinds[curgame], realIndex);

	// then reinitialize current game index :
	curinds[curgame] = -1;

	//look for the first main line move before (or equal) move asked :
	var firstMLmove = realIndex;
	while ( (games[curgame].charCodeAt(firstMLmove + 3) & 0x0F) > 0 ) firstMLmove -= 4;

	// now goto this move (shortcut)
	var savStepvar = stepvar[curgame];
	stepvar[curgame] = 0;
	while (curinds[curgame] < firstMLmove) {
		 Play(false, 0); // FALSE means "quiet", without visual moving 
	}

	// and move until asked one :
	var moveDepth = games[curgame].charCodeAt(realIndex + 3) & 0x0F;
	if (moveDepth > 0) stepvar[curgame] = 1;
	while (curinds[curgame] < realIndex)
		 Play(false, moveDepth); // FALSE means "quiet", without visual moving
	stepvar[curgame] = savStepvar;

	// now we can draw :
	DrawBoard();
}

// go to a specific move, from mouse click
function GotoMove_(gameNum, moveNum) {
	curgame = gameNum;
	GotoMove(moveNum);
}


/*******************
 * Flip board :
 ******************/

// exchange images by mirroring
//(more costly than simple redrawing, but needed for kibitz)
function SwapSources(i, j) {
	var tmpSrc = doc.images[ GetImagesIndex() + i ].src;
	doc.images[ GetImagesIndex() + i ].src = doc.images[ GetImagesIndex() + j ].src;
	doc.images[ GetImagesIndex() + j ].src = tmpSrc;
}
 
function FlipBoard() {
	// WARNING : if keystroke but no current game, nothing achieved
	if (curgame < 0) return;

	flipped[curgame] = 1 - flipped[curgame];
	if (kibitz) {
		//redraw by mirroring images URL's :
		for (var i=0; i<32; i++) SwapSources(i, 63-i);
	}
	else DrawBoard(); // normal drawing from internal representation
}

// call from control panel :
function FlipBoard_(gameNum) {
	curgame = gameNum;
	FlipBoard();
}


/*************************
 * Autoplay, on or off :
 ************************/

// infinite loop launched for automatic play :
function LoopAutoplay() {
	Play(true, 16); // 16 is larger than the max. move depth, to allow playing every variations
	timer = setTimeout("LoopAutoplay()", autotime);
}

// ..when autoplay is pressed again, it means stop :
function StopAutoplay() {
	clearTimeout(timer);
}

// main func. to set the timer which will call "Play()" every _autotime_ milliseconds
function Autoplay() {
	// WARNING : if keystroke but no current game, nothing achieved
	if (curgame < 0) return;

	autoply = 1 - autoply;
	if (autoply) LoopAutoplay();
	else StopAutoplay();
}

//..from mouse click :
function Autoplay_(gameNum) {
	if (kibitz) {
		DrawBoard(); // needed (not for all cases, but cleaner that way)
		kibitz = 0;
	}
	curgame = gameNum;
	Autoplay();
}


/****************************
 * Toggle stepvar variable :
 ***************************/

function StepVar() {
	// WARNING : if keystroke but no current game, nothing achieved
	if (curgame < 0) return;

	stepvar[curgame] = 1 - stepvar[curgame];
	// we need to redraw control panel
	doc.getElementById("cp"+curgame).src = (stepvar[curgame] ? ctrpv : ctrp);
}

// from mouse click :
function StepVar_(gameNum) {
	curgame = gameNum;
	StepVar();
}


/****************************
 * Handle keystrokes events :
 ***************************/

// search for '#' char in oldURL, beginning at the end ; stop at first slash !
/* change current page URL for TAB browsing */
function CutLastDiesis(url) {
	var i = url.length - 1;
	while (url[i] != '#' && url[i] != '/') i--;
	if (url[i] == '#') {
		// cut last part of URL :
		return url.substr(0,i);
	}
	return url;
}

/* called when keys down ;
 * code is :
 * - right arrow : play
 * - left arrow : undo
 * - w (AZERTY) / z (QWERTY) : GoBegin()
 * - x : BackFive()
 * - c : PlayFive()
 * - v : GoToEnd()
 * - SPACE : FlipBoard()
 * - BACK : Autoplay()
 * - DEL : step into variations
 * - TAB : change current game to n+1 mod. tot. games
 */
function HandleKeys(evt) {
	evt = (evt) ? evt : ((window.event) ? event : null);
	if (evt) {
		switch (evt.keyCode) {
		case 37: // left arrow :
			Back(true);
			break;
		case 39: // right arrow :
			Play(true, 16);
			break;
		case 87:
		case 90:
			// 'w' and 'z' are assigned to the same task, to handle azerty and qwerty keyboards
			GoBegin();
			break;
		case 88: // 'x' key
			BackFive();
			break;
		case 67: // 'c' key
			PlayFive();
			break;
		case 86: // 'v' key
			GoToEnd();
			break;
		case 32: // space bar
			evt.preventDefault();
			FlipBoard();
			break;
		case 8: // BACK key
			evt.preventDefault();
			Autoplay(); // si state à 0, on met à 1 et on joue ; sinon on met a zero et on stoppe
			break;
		case 46: // DEL key
			evt.preventDefault();
			StepVar();
			break;
		case 9: // TAB key
			evt.preventDefault();
			if (curgame < 0) curgame = 0;
			else curgame = (curgame + 1) % games.length;
			doc.location = CutLastDiesis(doc.location.href)+"#gm"+curgame;
			break;
		}
	}
}

