/********************************
 * functions for kibitz mode :
 ********************************/

//find the relative position of an object in the page
function findPos(obj) {
    var curleft = curtop = 0;
    if (obj.offsetParent) {
        do {
            curleft += obj.offsetLeft;
            curtop += obj.offsetTop;
        }
        while (obj = obj.offsetParent);
    }
    return [curleft,curtop];
}

// find the position of each board's upperleft coordinates
function InitCornersCoords() {
    for (var i=0; i < games.length; i++) {
        curgame = i;
		var index = GetImagesIndex();
        var pos = findPos(doc.images[index]);
        chcoords[i] = [ pos[0], pos[1] ];
    }
	curgame = -1;
}

//transform the pixels coordinates into a board number
function GetBoardFromCoords(xleft, ytop) {
	for (var i=0; i < games.length; i++) {
		if ( (chcoords[i][0] <= xleft && xleft <= chcoords[i][0] + 8 * sqsize)
				&& (chcoords[i][1] <= ytop && ytop <= chcoords[i][1] + 8 * sqsize) ) {
			return i;
		}
	}
	return -1;
}

// find indices corresponding to coordinates on board :
function GetMoveIndices() {
	var initIndex = 8 * (Math.floor( (initSquare[curgame][1] - chcoords[curgame][1]) / sqsize ))
				+ Math.floor( (initSquare[curgame][0] - chcoords[curgame][0]) / sqsize );
	var destIndex = 8 * (Math.floor( (destSquare[curgame][1] - chcoords[curgame][1]) / sqsize ))
				+ Math.floor( (destSquare[curgame][0] - chcoords[curgame][0]) / sqsize );
	return [initIndex, destIndex];
}

//piece code (as on boards[]) from image source :
function GetPieceCodeFromURL(url) {

	// we need to cut the URL to './printings/pieces/pieceType/sizeXX/pieceName.png'
	// we just count backslashes :
	var i = url.length - 1;
	var bcCount = 0;
	while (bcCount < 5) {
		if (url[i] == '/') bcCount++;
		i--;
	}
	cut_url = url.substr( i+1, url.length - (i+1) );
	cut_url = '.'+cut_url;
	
	if (cut_url==bp) return -1;
	if (cut_url==wp) return 1;
	if (cut_url==br) return -2;
	if (cut_url==wr) return 2;
	if (cut_url==bn) return -3;
	if (cut_url==wn) return 3;
	if (cut_url==bb) return -4;
	if (cut_url==wb) return 4;
	if (cut_url==bq) return -5;
	if (cut_url==wq) return 5;
	if (cut_url==bk) return -6;
	if (cut_url==wk) return 6;
	return 0;
}

// go to beginning for kibitz mode (free moves' stack)
function GoBegin_kibitz() {
	var lg = stackMoves[curgame].length;
	for (var i = 0; i < lg; i++) Back_kibitz();
}

// back five kibitzed moves
function BackFive_kibitz() {
	for (var i=0; i<10; i++) Back_kibitz();
}

// back one kibitzed move :
function Back_kibitz() {
	if (stackMoves[curgame].length == 0) return;
	var lastMove = stackMoves[curgame].pop();
	Back_coords(lastMove[0], lastMove[1], lastMove[2], false, true);
}

// uses current game number and move coordinates to check if this is legal :
// BASIC check : moves which let the kiong in check can be played.. (then the king can be taken !)
//so little change : small castle represented by lowest bits 111, and big castle with highest bit (0x80)
function CheckMove() {

	// first get move indices :
	var inds = GetMoveIndices();
	var piece = GetPieceCodeFromURL( doc.images[ GetImagesIndex() + inds[0] ].src );
	var destSquare = GetPieceCodeFromURL( doc.images[ GetImagesIndex() + inds[1] ].src );
	if ( piece == 0 || (piece > 0 && destSquare > 0) || (piece < 0 && destSquare < 0) ) return 0;
	if (Math.abs(destSquare) == 6) return 0; // no take king
	
	var flags = ( 0x40 | Math.abs(destSquare) );
	var initFile = inds[0]%8;
	var initRank = (inds[0]-initFile)/8;
	var destFile = inds[1]%8;
	var destRank = (inds[1]-destFile)/8;

	//what do we find at the initial square ?
	
	if ( (piece==1 && !flipped[curgame]) || (piece==-1 && flipped[curgame]) ) {
		if (inds[1] < 8) flags |= 0x10; // promotion ; will pop a modal box
		if ( destSquare==0 && ( (inds[1] == inds[0]-8) || ( (inds[0] >= 48) && (inds[1] == inds[0]-16)
				&& (GetPieceCodeFromURL( doc.images[ GetImagesIndex() + inds[0]-8 ].src ) == 0) ) ) ) return flags;
		if ( ( (inds[1] == inds[0]-7) && (destFile == initFile+1) )
				|| ( (inds[1] == inds[0]-9) && (destFile == initFile-1) ) ) {
			if ( (piece>0 && destSquare<0) || (piece<0 && destSquare>0) ) return flags; //simple capture
			if ( (initRank==3) &&
					( (piece>0 && GetPieceCodeFromURL( doc.images[ GetImagesIndex() + inds[1]+8 ].src ) == -1) ||
					(piece<0 && GetPieceCodeFromURL( doc.images[ GetImagesIndex() + inds[1]+8 ].src ) == 1) ) )
				return (flags | 0x28); //en passant
		}
		return 0;
	}
	if ( (piece==1 && flipped[curgame]) || (piece==-1 && !flipped[curgame]) ) {
		if (inds[1] >= 56) flags |= 0x10; // promotion ; will pop a modal box
		if ( destSquare==0 && ( (inds[1] == inds[0]+8) || ( (inds[0] < 16) && (inds[1] == inds[0]+16)
				&& (GetPieceCodeFromURL( doc.images[ GetImagesIndex() + inds[0]+8 ].src ) == 0) ) ) ) return flags;
		if ( ( (inds[1] == inds[0]+7) && (destFile == initFile-1) )
				|| ( (inds[1] == inds[0]+9) && (destFile == initFile+1) ) ) {
			if ( (piece>0 && destSquare<0) || (piece<0 && destSquare>0) ) return flags; //simple capture
			if ( (initRank==4) &&
					( (piece>0 && GetPieceCodeFromURL( doc.images[ GetImagesIndex() + inds[1]-8 ].src ) == -1) ||
					(piece<0 && GetPieceCodeFromURL( doc.images[ GetImagesIndex() + inds[1]-8 ].src ) == 1) ) )
				return (flags | 0x28); //en passant
		}
		return 0;
	}

	switch (piece) {
	
	case 2:
	case -2:
		// check if this is a valid rook movement :
		if (initFile == destFile) {
			// file movement :
			var m = Math.min(initRank, destRank);
			var M = Math.max(initRank, destRank);
			for (var i=m+1; i<M; i++) {
				if ( GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 8*i+initFile ].src ) != 0 ) return 0;
			}
			return flags;
		}
		else if (initRank == destRank) {
			// rank movement :
			var m = Math.min(initFile, destFile);
			var M = Math.max(initFile, destFile);
			for (var i=m+1; i<M; i++) {
				if ( GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 8*initRank+i ].src ) != 0 ) return 0;
			}
			return flags;
		}
		return 0;
		
	case 3:
	case -3:
		//check if this is a valid knight movement :
		if ( (piece > 0 && destSquare > 0) || (piece < 0 && destSquare < 0) ) return 0;
		if ( (Math.abs(initFile - destFile) == 2 && Math.abs(initRank - destRank) == 1)
				|| (Math.abs(initFile - destFile) == 1 && Math.abs(initRank - destRank) == 2) ) {
			return flags;
		}
		return 0;
	
	case 4:
	case -4:
		// check if this is a valid bishop movement :
		var miRank = 7 - initRank;
		var mdRank = 7 - destRank;
		var deltaF = Math.abs(initFile - destFile);
		if ( (initFile-initRank) == (destFile-destRank) ) {
			// move along "opposite diagonal" :
			mr = Math.min(initRank, destRank);
			mf = Math.min(initFile, destFile);
			for (var i=1; i<deltaF; i++) {
				if ( GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 8*(mr+i)+mf+i ].src ) != 0 ) return 0;
			}
			return flags;
		}
		if ( (initFile-miRank) == (destFile-mdRank) ) {
			// move along "main diagonal" :
			mr = Math.min(initRank, destRank);
			mf = Math.max(initFile, destFile);
			for (var i=1; i<deltaF; i++) {
				if ( GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 8*(mr+i)+mf-i ].src ) != 0 ) return 0;
			}
			return flags;
		}
		return 0;
		
	case 5:
	case -5:
		// check if this is a valid queen movement : rook + bishop
		var miRank = 7 - initRank;
		var mdRank = 7 - destRank;
		var deltaF = Math.abs(initFile - destFile);
		if (initFile == destFile) {
			// file movement :
			var m = Math.min(initRank, destRank);
			var M = Math.max(initRank, destRank);
			for (var i=m+1; i<M; i++) {
				if ( GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 8*i+initFile ].src ) != 0 ) return 0;
			}
			return flags;
		}
		if (initRank == destRank) {
			// rank movement :
			var m = Math.min(initFile, destFile);
			var M = Math.max(initFile, destFile);
			for (var i=m+1; i<M; i++) {
				if ( GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 8*initRank+i ].src ) != 0 ) return 0;
			}
			return flags;
		}
		else if ( (initFile-initRank) == (destFile-destRank) ) {
			// move along "opposite diagonal" :
			mr = Math.min(initRank, destRank);
			mf = Math.min(initFile, destFile);
			for (var i=1; i<deltaF; i++) {
				if ( GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 8*(mr+i)+mf+i ].src ) != 0 ) return 0;
			}
			return flags;
		}
		else if ( (initFile-miRank) == (destFile-mdRank) ) {
			// move along "main diagonal" :
			mr = Math.min(initRank, destRank);
			mf = Math.max(initFile, destFile);
			for (var i=1; i<deltaF; i++) {
				if ( GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 8*(mr+i)+mf-i ].src ) != 0 ) return 0;
			}
			return flags;
		}
		return 0;

	case 6:
	case -6:
		// check if this is a valid king movement :
		if ( Math.abs(initFile-destFile) <= 1 && Math.abs(initRank-destRank) <= 1 ) return flags;
		//check small castle :
		if ( (piece > 0 && inds[0] == 60 && inds[1] == 62
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 63 ].src ) == 2
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 62 ].src ) == 0
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 61 ].src ) == 0
			 ) ||
			 (piece < 0 && inds[0] == 4 && inds[1] == 6
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 7 ].src ) == -2
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 6 ].src ) == 0
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 5 ].src ) == 0
			)) {
			return (flags | 0x06);
		}
		// check big castle :
		if ( (piece > 0 && inds[0] == 60 && inds[1] == 58
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 56 ].src ) == 2
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 57 ].src ) == 0
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 58 ].src ) == 0
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 59 ].src ) == 0
			 ) ||
			 (piece < 0 && inds[0] == 4 && inds[1] == 2
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 0 ].src ) == -2
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 1 ].src ) == 0
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 2 ].src ) == 0
					&& GetPieceCodeFromURL( doc.images[ GetImagesIndex() + 3 ].src ) == 0
			)) {
			return (flags | 0x07);
		}
		return 0;
		
	}
	return 0;
}

//play the move (only visual !)
function Play_kibitz(flags) {
	kibitz = 1; // says "OK the user started kibitzing ; we will redraw board after"

	// now get move indices :
	var inds = GetMoveIndices();
	if (flipped[curgame]) {
		inds[0] = 63 - inds[0];
		inds[1] = 63 - inds[1];
	}

	//and play it:
	Play_coords(inds[0], inds[1], flags, false, true);

	// and push it in the moves stack :
	stackMoves[curgame].push([inds[0],inds[1],flags]);
}

//turn off initSquare on current board :
function UnHighlight() {
	var inds = GetMoveIndices();
	if (inds[0] < 0) return;
	if (inds[0] >= 16) {
		doc.images[ GetImagesIndex() + inds[0] ].style.backgroundColor =
		doc.images[ GetImagesIndex() + inds[0]-16 ].style.backgroundColor;
	}
	else if (inds[0] < 48) {
		doc.images[ GetImagesIndex() + inds[0] ].style.backgroundColor =
		doc.images[ GetImagesIndex() + inds[0]+16 ].style.backgroundColor;
	}
}

//turn off initSquare on each board (back to normal play) :
function UnHighlight_all() {
	var savCurgame = curgame;
	for (var i=0; i < games.length; i++) {
		curgame = i;
		UnHighlight();
	}
	curgame = savCurgame;
}

// highlight curgame initial square :
function Highlight_kibitz() {
	var inds = GetMoveIndices();
	doc.images[ GetImagesIndex() + inds[0] ].style.backgroundColor = bgsqcol;
}

//reinitialize graphic variables :
function ReinitVars() {
	UnHighlight();
	initSquare[curgame] = [-1,-1];
	destSquare[curgame] = [-1,-1];
}

// show a modal box to choose the promoted piece :
function Promotion(flags, color) {

	flags ^= 0x10; //needed if not knight/bishop promotion
	var wleft = Math.floor(screen.width/3);
	var wtop = Math.floor(screen.height/3);
	var wwidth = Math.max(6*sqsize, 100);
	var wheight = Math.max(3*sqsize, 100);
	var args='width='+wwidth+',height='+wheight+',left='+wleft+',top='+wtop+',directories=0';
		args += 'location=0,menubar=0,resizable=0,scrollbars=0,status=0,menubar=0,toolbar=0';

	var PromoteWindow=window.open("","",args);
	var wprom = PromoteWindow.document;
	var offsetTop = Math.floor( (wprom.body.clientHeight - sqsize) / 2 );
	wprom.open();
	wprom.write('<!DOCTYPE html>');
	wprom.write('<html>');
	wprom.write('<head>');
	wprom.write('<title>Promotion</title>');
	wprom.write('<style>');
	wprom.write('	div {');
	wprom.write('		margin-top: '+offsetTop+'px;');
	wprom.write('	}');
	wprom.write('	table {');
	wprom.write('		border-collapse: collapse;');
	wprom.write('		margin: 0 auto;');
	wprom.write('	}');
	wprom.write('</style>');
	wprom.write('<script' + ' type="text/javascript">');
	wprom.write('	function CloseForm(code) {');
	wprom.write('		window.opener.Play_kibitz('+flags+' | code);');
	wprom.write('		window.close();'); // and close promotion window
	wprom.write('	} ');
	wprom.write('</script' + '>');       
	wprom.write('</head>');  
	wprom.write('<body>');
	if (color) {
		wprom.write('<div><table><tr>');
		wprom.write('<td><img src='+wr+' onClick="CloseForm(0x08); window.opener.ReinitVars()"></td>');
		wprom.write('<td><img src='+wn+' onClick="CloseForm(0x10); window.opener.ReinitVars()"></td>');
		wprom.write('<td><img src='+wb+' onClick="CloseForm(0x18); window.opener.ReinitVars()"></td>');
		wprom.write('<td><img src='+wq+' onClick="CloseForm(0x20); window.opener.ReinitVars()"></td>');
		wprom.write('</tr></table></div>');
	}
	else {
		wprom.write('<div><table><tr>');
		wprom.write('<td><img src='+br+' onClick="CloseForm(0x08); window.opener.ReinitVars()"></td>');
		wprom.write('<td><img src='+bn+' onClick="CloseForm(0x10); window.opener.ReinitVars()"></td>');
		wprom.write('<td><img src='+bb+' onClick="CloseForm(0x18); window.opener.ReinitVars()"></td>');
		wprom.write('<td><img src='+bq+' onClick="CloseForm(0x20); window.opener.ReinitVars()"></td>');
		wprom.write('</tr></table></div>');
	}
	wprom.write('</body>');
	wprom.write('</html>');
	wprom.close();
	wprom.focus();
}

// catching mouse events :
function OnMouseClick(evt) {
	evt = (evt) ? evt : ((window.event) ? event : null);
	if (evt) {

        // first determine if this is a mouse click on some chessboard :
        var xleft = evt.pageX;
        var ytop = evt.pageY;
        var brd = GetBoardFromCoords(xleft, ytop);
        if (brd < 0) return;

        //if yes, then set curgame and remember click
		curgame = brd;
        if (initSquare[brd][0] == -1) {
        	initSquare[brd] = [xleft, ytop];
			Highlight_kibitz();
		}
		
		else {
			destSquare[brd] = [xleft, ytop];
			// now proceed :
			var flags = CheckMove();
			if (flags > 0) {
				if (flags & 0x10) {
					//will ask for the promoted piece, then play and reinit:
					var inds = GetMoveIndices();
					var color = (inds[1] < 8 ? 1 : 0);
					Promotion(flags, color);
				}
				else Play_kibitz(flags);
			}
			ReinitVars(); //reinit variables
		}
	}
}

