/*******************************
 * Play button or keystroke :
 *******************************/

// get file path of promoted piece, according to flag and color
function PathToPPiece(highBits, color) {
	switch (highBits) {
	case 0x08: return (color>0 ? wr : br);
	case 0x10: return (color>0 ? wn : bn);
	case 0x18: return (color>0 ? wb : bb);
	case 0x20: return (color>0 ? wq : bq);
	}
	return "";
}

//..and corresponding piece code
function CodePiece(promoteFlag) {
	switch (promoteFlag) {
	case 0x08: return 2;
	case 0x10: return 3;
	case 0x18: return 4;
	case 0x20: return 5;
	}
	return 0;
}

// play a move on specified board (intern, visual)
function Play_coords(initSquare, destSquare, flags, intern, visual) {

	// do basic move on [internal] [and visual] board :
	if (intern) BasicMoveInternal(initSquare, destSquare);
	if (visual) BasicMoveHTML(initSquare, destSquare);

	// ..and complete move if needed :

	// castle & capture flags
	var lowBits = flags & 0x07;
	if (lowBits == 6) {
		// small castle : move rook
		if (intern) BasicMoveInternal(destSquare+1, initSquare+1);
		if (visual) BasicMoveHTML(destSquare+1, initSquare+1);
	}
	else if (lowBits == 7) {
		// big castle : move rook
		if (intern) BasicMoveInternal(destSquare-2, initSquare-1);
		if (visual) BasicMoveHTML(destSquare-2, initSquare-1);
	}
	// nothing to do in case of capture here

	// promotion & en passant flags
	var highBits = flags & 0x38;
	if (highBits == 0x28) {
		// en passant : delete opponent pawn
		// first we must know if this is a white or black move.
		var colorShift = (initSquare > destSquare ? 8 : -8);
		if (intern) boards[curgame][destSquare+colorShift] = 0;
		if (visual) SetOnHTML(destSquare+colorShift, empty);
	}
	else if (highBits > 0) {
		// promotion :
		var color = (initSquare > destSquare ? 1 : -1); // 1 for white
		if (intern) boards[curgame][destSquare] = color * CodePiece(highBits);
		if (visual) SetOnHTML(destSquare, PathToPPiece(highBits, color));
	}
}

// play the move on internal [and HTML] board (MAIN call)
function BoardPlay(index, visual) {

	//first, get initial and destination squares ;
	// ascii to numeric (from up left to down right) :
	var initSquare = games[curgame].charCodeAt(index) - 63;
	var destSquare = games[curgame].charCodeAt(index + 1) - 63;

	// get move flags : ascii to numeric
	var flags = games[curgame].charCodeAt(index + 2);

	// then call main "play" function ;
    // reason for this separation = possible kibitz mode !
	Play_coords(initSquare, destSquare, flags, true, visual);
}

/* this function takes the current index as parameter,
 * and moves to the left until depth <= refDepth
 * meanwhile, it back "smartly" the moves met
 */
function SmartBack(index, refDepth) {

	var initDepth = (games[curgame].charCodeAt(index + 3) & 0x0F);
	var curInd = index;
	var curDepth = initDepth;
	var justQuitVar=false; //meaning == we quit var AND we backed it
	var justQuitCont=false; //meaning == we quit cont. AND we backed it
	var depth = initDepth; // moving depth reference -- decreasing

	while (curDepth > refDepth) {

		if ( curDepth == depth ) {
			if (! (games[curgame].charCodeAt(curInd + 3) & 0x20) ) {
				justQuitVar = false;
				BoardBack(curInd, false);
				curInd -= 4;
			}
			else {
				justQuitVar = true;
				justQuitCont = (games[curgame].charCodeAt(curInd + 3) & 0x10);
				BoardBack(curInd, false); // back it in every case
				while ( (games[curgame].charCodeAt(curInd - 1) & 0x0F) >= depth) curInd -= 4; // skip the (...)(... case
				curInd -= 4;
			}
		}

		else if (curDepth < depth) {
			if ( !justQuitVar || justQuitCont ) 
				BoardBack(curInd, false);
			curInd -= 4;
			depth = curDepth;
		}

		else if (curDepth > depth) {
			var remFromDepth = 0;
			if (games[curgame].charCodeAt(curInd + 7) & 0x20) remFromDepth = 1;
			while ( (games[curgame].charCodeAt(curInd - 1) & 0x0F) > (depth - remFromDepth) ) curInd -= 4; // skip var.
			curInd -= 4;
		}

		curDepth = games[curgame].charCodeAt(curInd + 3) & 0x0F;
	}
	return curInd;
}

// this function just plays a move after a variation, given specific conditions :
function PlayNextMove(lastInd, index, nextInd) {

    // find the exact beginning of exiting variation
	var vbegin = index;
	while ( !(games[curgame].charCodeAt(vbegin+3) & 0x20) ) vbegin -= 4;

    if (games[curgame].charCodeAt(nextInd+3) & 0x20) {
        if ( (games[curgame].charCodeAt(nextInd+3) & 0x10) && !(games[curgame].charCodeAt(vbegin+3) & 0x10) ) {
            // play again shadowed move :
            BoardPlay(lastInd, false);
        }
        else if ( !(games[curgame].charCodeAt(nextInd+3) & 0x10) && (games[curgame].charCodeAt(vbegin+3) & 0x10) ) {
            // back again shadowed move :
            BoardBack(lastInd, false);
        }
    }
    else {
        if ( !(games[curgame].charCodeAt(vbegin+3) & 0x10) ) {
            // play again shadowed move :
            BoardPlay(lastInd, false);
        }
    }

    //play next move to the right :
    BoardPlay(nextInd, false);
}

/********************************************************
 * main function to play a move ;
 * its first argument indicates if visual moving or not,
 * while the second argument says "don't go deeper than maxDepth"
 * it works with global variables
 *******************************************************/
function Play(visual, maxDepth) {
	// WARNING : if keystroke but no current game, nothing achieved
	if (curgame < 0) return;

	var index = curinds[curgame]; // convention : this index point to the last move played
								  // (-1 if nothing played)
	// WARNING : if we are at the end of the game, do nothing
	if (index == games[curgame].length - 4) return;

	// if first move in the game.. :
	if (index < 0) {
		BoardPlay(0, visual);
		if (visual) Highlight(-1, 0);
		curinds[curgame] = 0;
		return;
	}

	var vdepth = games[curgame].charCodeAt(index + 3);
	var nextVD = games[curgame].charCodeAt(index + 7);
	var depth = vdepth & 0x0F;
	var nextDepth = nextVD & 0x0F;

	// we get rid of the simple case first :

	if (depth == nextDepth && ! (nextVD & 0x20) ) { //NORMAL MOVING
		BoardPlay(index+4, visual);
		if (visual) Highlight(index, index+4);
		curinds[curgame] += 4;
	}

	else if (depth < nextDepth) { // BEGINNING OF NEW VARIATION
		if (stepvar[curgame] && nextDepth <= maxDepth) {
			if ( !(nextVD & 0x10) ) BoardBack(index, visual);
			BoardPlay(index+4, visual);
			if (visual) Highlight(index, index+4);
			curinds[curgame] += 4;
		}
		else {
			var nextInd = index;
			var outDepth = nextDepth;
			while (outDepth >= nextDepth) {
				nextInd += 4;
				if (nextInd == games[curgame].length) break;
				outDepth = games[curgame].charCodeAt(nextInd + 3) & 0x0F;
			}

			if (nextInd < games[curgame].length) {
				if (outDepth == depth && ! (games[curgame].charCodeAt(nextInd + 3) & 0x20) ) {
					//easy case: outDepth == depth and nextInd move isn't beginning of var.
					//play next move to the right :
					BoardPlay(nextInd, visual);
					if (visual) Highlight(index, nextInd);
					curinds[curgame] = nextInd;
				}

				else {
					// hard case : some additional parenthesis..
					var remFromDepth = ( (games[curgame].charCodeAt(nextInd + 3) & 0x20) ? 1 : 0 );
					var lastInd = SmartBack(index, outDepth-remFromDepth);
					PlayNextMove(lastInd, index, nextInd);
				    if (visual) {
						DrawBoard();
						Highlight(index, nextInd);
					}
					curinds[curgame] = nextInd;
				}
			}
		}
	}

	else { // 1..n CLOSING PARENTHESIS, 0 or 1 OPENED then
		var remFromDepth = ( (nextVD & 0x20) ? 1 : 0 );
		var lastInd = SmartBack(index, nextDepth - remFromDepth);
        PlayNextMove(lastInd, index, index+4);
		if (visual) {
			DrawBoard();
			Highlight(index, index+4);
		}
		curinds[curgame] = index+4;
	}
	
	// if we just played a NULL move, play again :
	if ( games[curgame].charCodeAt(curinds[curgame]) == games[curgame].charCodeAt(curinds[curgame]+1) )
		Play(visual, maxDepth);
}

// in case of a click on a control panel, the game index is known :
function Play_(gameNum) {
	curgame = gameNum;
	if (kibitz) {
		//clean board :
		UnHighlight_all();
		GoBegin_kibitz();
		DrawBoard();
		kibitz = 0;
	}
	Play(true, 16); // true means "normal" moving, with HTML redrawing
                    // 16 means maxDepth==infinity, because can't be more than 14
}


/************************************
 * PlayFive button or keystroke :
 ************************************/

// play 5 entire moves (10 half moves)
function PlayFive() {
	// WARNING : if keystroke but no current game, nothing achieved
	if (curgame < 0) return;

	var turnoff = curinds[curgame];
	for (var i=0; i<10; i++) Play(false, 16);
	// redraw board and highlight last move :
	DrawBoard();
	Highlight(turnoff, curinds[curgame]);
}

//when click on control panel
function PlayFive_(gameNum) {
	curgame = gameNum;
	PlayFive();
}


/********************************
 * GoToEnd button or keystroke :
 ********************************/

// play every move to the end, and draw the result
function GoToEnd() {
	// WARNING : if keystroke but no current game, nothing achieved
	if (curgame < 0) return;

	//look for the last main line move :
	var lastMLmove = games[curgame].length - 4;
	while ( (games[curgame].charCodeAt(lastMLmove + 3) & 0x0F) > 0 ) lastMLmove -= 4;

	//Goto It :

	var turnoff = curinds[curgame];
	var savStepvar = stepvar[curgame];
	stepvar[curgame] = 0; // skip variations

	// first case, we are before it :
	if (curinds[curgame] < lastMLmove) {
		while (curinds[curgame] < lastMLmove) Play(false, 16);
	}
	// else we are after it :
	else {
		while (curinds[curgame] > lastMLmove) Back(false, 16);
	}

	stepvar[curgame] = savStepvar;
	DrawBoard();
	// finally, highlight last move
	Highlight(turnoff, curinds[curgame]);
}

// control panel click :
function GoToEnd_(gameNum) {
	curgame = gameNum;
	GoToEnd();
}

