/********************************
 * GoBegin button or keystroke :
 ********************************/

// main function to redraw the initial board, internal and HTML
// it also reset game current index.
function GoBegin() {
	// WARNING : if keystroke but no current game, nothing achieved
	if (curgame < 0) return;

	//reset and draw from internal board :
	ResetBoard();
	DrawBoard();

	// take care of printing/highlighting :
	Highlight(curinds[curgame] - 4, -1);

	curinds[curgame] = -1; // reinitialize current index in game
}

//when mouse click on control panel :
function GoBegin_(gameNum) {
	curgame = gameNum;
	if (kibitz) GoBegin_kibitz();
	else GoBegin();
}


/************************************
 * BackFive button or keystroke :
 ************************************/

// takeback 5 entire moves (10 half moves)
function BackFive() {
	// WARNING : if keystroke but no current game, nothing achieved
	if (curgame < 0) return;

	var index = curinds[curgame];
	for (var i=0; i<10; i++) Back(false); // false for quiet, no printings
	// now redraw board :
	DrawBoard();
	Highlight(index, curinds[curgame]);
}

//..on mouse click :
function BackFive_(gameNum) {
	curgame = gameNum;
	if (kibitz) BackFive_kibitz();
	else BackFive();
}


/*******************************
 * Back button or keystroke :
 *******************************/
 
// takeback the move on [internal] [and HTML] board (SECONDARY call) ;
// in case of kibitz mode, this function has to be separated.
function Back_coords(initSquare, destSquare, flags, intern, visual) {

	// do basic undo on internal board :
	if (intern) BasicMoveInternal(destSquare, initSquare);
	if (visual) BasicMoveHTML(destSquare, initSquare);

	// ..and complete move if needed :

	// castle & capture flags
	var lowBits = flags & 0x07;
	if (lowBits == 6) {
		// small castle : move rook
		if (intern) BasicMoveInternal(initSquare+1, destSquare+1);
		if (visual) BasicMoveHTML(initSquare+1, destSquare+1);
	}
	else if (lowBits == 7) {
		// big castle : move rook
		if (intern) BasicMoveInternal(initSquare-1, destSquare-2);
		if (visual) BasicMoveHTML(initSquare-1, destSquare-2);
	}

	// in case of capture : give back captured piece
	else if (lowBits > 0) {
		var color = 0;
		if (intern) color = (boards[curgame][initSquare]>0 ? -1 : 1);
		else if (visual) {
			var ifFlipped = 0;
			if (flipped[curgame]) ifFlipped = 63 - 2*initSquare;
			url = doc.images[GetImagesIndex() + initSquare + ifFlipped].src;
			color = ( url[url.length - 6] == 'w' ? -1 : 1 );
		}
		if (intern) boards[curgame][destSquare] = lowBits * color;
		if (visual) SetOnHTML(destSquare, GetPieceURL(lowBits * color, false));
	}

	// promotion & en passant flags
	var highBits = flags & 0x38;
	if (highBits == 0x28) {
		// en passant : give back opponent pawn
		// first we must know if this is a white or black move.
		var color = (initSquare > destSquare ? 1 : -1);
		if (intern) boards[curgame][destSquare+8*color] = -color;
		if (visual) SetOnHTML( destSquare+8*color, (color>0 ? bp : wp) );
	}
	else if (highBits > 0) {
		// promotion :
		var color = (initSquare > destSquare ? 1 : -1); // 1 for white
		if (intern) boards[curgame][initSquare] = color;
		if (visual) SetOnHTML( initSquare, (color>0 ? wp : bp) );
	}
}

// takeback the move on internal [and HTML] board (MAIN call)
function BoardBack(index, visual) {

	//first, get initial and destination squares ;
	// ascii to numeric (from up left to down right) :
	var initSquare = games[curgame].charCodeAt(index) - 63;
	var destSquare = games[curgame].charCodeAt(index + 1) - 63;

	//and get move flags : ascii to numeric
	var flags = games[curgame].charCodeAt(index + 2);

	// then call main "back" function ;
    // reason for this separation = possible kibitz mode !
	Back_coords(initSquare, destSquare, flags, true, visual);
}

/********************************************************
 * main function to takeback a move ;
 * its only argument indicates if visual moving or not ;
 * it works with global variables
 *******************************************************/
function Back(visual) {
	// WARNING : if keystroke but no current game, nothing achieved
	if (curgame < 0) return;

	var index = curinds[curgame];
	if (index < 0) return; // nothing to takeback, because nothing played !

	// WARNING : if we are at the beginning of the game just cancel this move
	// and retablish "initial position" highlighting
	if (index == 0) {
		BoardBack(0, visual);
		if (visual) Highlight(0, -1);
		curinds[curgame] = -1;
		return;
	}

	var vdepth = games[curgame].charCodeAt(index + 3);
	var lastVD = games[curgame].charCodeAt(index - 1);
	var depth = vdepth & 0x0F;
	var lastDepth = lastVD & 0x0F;

	// we get rid of the simple case first :

	if (depth == lastDepth && !(vdepth & 0x20) ) { // NORMAL MOVING
		BoardBack(index, visual);
		if (visual) Highlight(index, index-4);
		curinds[curgame] = index-4;
	}

	else if (depth > lastDepth) { // ONE OPENING PARENTHESIS -- watch out the cases (x(y(z..,  (x))(y(z ... !
		BoardBack(index, visual);
		var curInd = index;

		// while last index is lonely beginning of var (NO CONT.!) without closing parenthesis ; case x(y(z(...
		while ( (games[curgame].charCodeAt(curInd - 1) & 0x20) && !(games[curgame].charCodeAt(curInd - 1) & 0x10)
				&& ((games[curgame].charCodeAt(curInd - 1) & 0x0F) == (games[curgame].charCodeAt(curInd + 3) & 0x0F) - 1) )
			curInd -= 4;

		// now, we know that the move right before curInd has not to be taken back (unless cont...), and
		// is not preceded by a lonely beginning of variation ; so we are at z :
		// x X y ( z where X can be : nothing or "opening cont." (we are happy then) -- but pay attention if y is the game beginning !
		//							  some closing parenthesis, or
		//							  some closing par. + one opening !

		//get rid of simple cases :
        if (curInd==4 || (games[curgame].charCodeAt(curInd + 3) & 0x10)) curinds[curgame] = curInd - 4;
        else if ( (games[curgame].charCodeAt(curInd - 5) & 0x0F) > (games[curgame].charCodeAt(curInd - 1) & 0x0F) ) {
            BoardPlay(curInd-4, visual); //play again shadowed move
            curinds[curgame] = curInd - 4;
        }
		else if ( curInd==index || ( (games[curgame].charCodeAt(curInd - 5) & 0x0F) == (games[curgame].charCodeAt(curInd - 1) & 0x0F) )
				&& !(games[curgame].charCodeAt(curInd - 1) & 0x20) ) {
			curinds[curgame] = curInd - 8;
		}
		
		else {
			// either x )..)( y ( z or x )..) y ( z
			// in both cases we need to skip from y :

			// find the right point TO THE LEFT
			var lastInd = curInd - 8;
			var lastI_depth = (games[curgame].charCodeAt(lastInd + 3) & 0x0F);
			var curDepth = lastI_depth;
			var depth = (games[curgame].charCodeAt(curInd - 1) & 0x0F);
			var remFromDepth = ( (games[curgame].charCodeAt(curInd - 1) & 0x20) ? 1 : 0 );
			while (curDepth > depth-remFromDepth) {
				curDepth = (games[curgame].charCodeAt(lastInd - 1) & 0x0F);
				lastInd -= 4;
			}
			curinds[curgame] = lastInd;
            //if not continuation, we play again move before var :
            if ( remFromDepth && !(games[curgame].charCodeAt(curInd - 1) & 0x10) ) BoardPlay(lastInd, visual);
            // cases )))... OR )))(... : enter only if stepvar AND not too deep
            // play until correct move reached :
            if (stepvar[curgame]) {
                while (curinds[curgame] < curInd-8) Play(false, lastI_depth);
            }
		}

		if (visual && stepvar[curgame]) DrawBoard();
		if (visual) Highlight(index, curinds[curgame]);
	}

	else { // 1..n CLOSING PARENTHESIS, 0 or 1 OPENED then [backward]
		   // cases : )( or )))( or )))...
		BoardBack(index, visual);

		// find the right point TO THE LEFT
		var lastInd = index;
		var curDepth = lastDepth;
		var remFromDepth = ( (vdepth & 0x20) ? 1 : 0 );
		while (curDepth > depth-remFromDepth) {
			curDepth = (games[curgame].charCodeAt(lastInd - 1) & 0x0F);
			lastInd -= 4;
		}
		curinds[curgame] = lastInd;
        //if not continuation, we play again move before var :
        if ( remFromDepth && !(vdepth & 0x10) ) BoardPlay(lastInd, visual);
        // cases )))... OR )))(... : enter only if stepvar AND not too deep
        // play until correct move reached :
        if (stepvar[curgame]) {
            while (curinds[curgame] < index-4) Play(false, lastDepth);
        }

		if (visual && stepvar[curgame]) DrawBoard();
		if (visual) Highlight(index, curinds[curgame]);
	}
	
	// if we fall on a NULL move, back again :
	if ( games[curgame][curinds[curgame]] == games[curgame][curinds[curgame]+1] )
		Back(visual);
}

// from mouse click on control panel :
function Back_(gameNum) {
	curgame = gameNum;
	if (kibitz) Back_kibitz();
	else Back(true);
}

