<?php

// "configuration file" containing some default values :
include ("viewer_config.php");

// function to get a variable value from its name :
function getVar($varname) {
	global $allVars;
	if (isset($_GET[$varname])) return ($_GET[$varname]);
	return ($allVars[$varname]);
}

// function to check if a square size is allowed :
function checkSize($size, $enableWrite) {
	// first check to avoid foolish sizes because of typing errors :
	if ($size < 10) $size = 10;
	else if ($size > 100) $size = 100;
	if ( ! $enableWrite ) {
		// find the nearest integer in the list 10-20-30-40-50-60-70-80-90-100 (and prepare all images before !)
		$mod10 = $size % 10;
		$size += ( ($mod10 < 5) ? - $mod10 : (10 - $mod10) );
		if ($size < 10) $size = 10;
		else if ($size > 100) $size = 100;
	}
	return $size;
}

// GET SOME IMPORTANT GLOBAL VARIABLES :
$pieceStyle = getVar("psty");
$squareSize = checkSize( getVar("sqz"), $allVars["wrenable_"] );
$boardSize = 8 * $squareSize;
$lcol = getVar("lcol","fffff0");
$dcol = getVar("dcol","29A197");

// CREATE IMAGES FILES AND DIRECTORIES IF NEEDED :
include ("viewer_cimages.php");
$ep = getElemsPaths($squareSize, $pieceStyle, $allVars["simpctrp_"]); // ep like "elements paths" (pieces, empty, control..)
if ($allVars["wrenable_"]) createElemsImages($ep, $squareSize, $pieceStyle, $allVars["simpctrp_"]);

// LAUNCH MAIN PARSING :
include ("viewer_parsepgn.php");
$argFileName = getopt('f:'); // possible for command line only !
/* get games file name :
 * can be set by 2 ways : viewer_config.php, or command-line / _GET argument
 * default output name is games.html, but can change..
 */
$gamesFileName = $argFileName['f'];
if ($gamesFileName == "") $gamesFileName = getVar("file");
$gamesFileName = "../".$gamesFileName;
$sg = parseAllGames($gamesFileName, $allVars["nagsfont_"]); // sg like "strings (representing) games"
$totGames = count($sg["asciiMoves"]);


/************************************************
*************************************************/
// BEGIN HTML DOCUMENT :


// style global variables :
$padding = 15;
$bwidth = 2;

echo '<!DOCTYPE html>
<html>
<head>

<title>generated by dynachess v1.0</title>
<link rel="stylesheet" type="text/css" href="./sources/dcstyles.css" />

<!-- a few styles for the current document, which have to be there because dynamically generated by viewer.php -->
<style>
	table.chessboard td.l {
		background-color: #'.$lcol.';
	}
	table.chessboard td.d {
		background-color: #'.$dcol.';
	}
	.vars {
		/* style for variations (font,color,..) */
		color: #0000A0;';
if ($allVars["varout_"]>0) {
	echo '
		/* this is to indent variations properly */
		margin-left: '; echo (2 * $padding); echo 'px;
		display: block;';
}
echo '
	}';
echo '
	.container { /* the div that contains the floating elements */
	    min-width: '; echo ($boardSize + 200); echo 'px;
		border-width : '; echo $bwidth; echo "px;
		border-color : #0000A0;
		border-style : solid;
		margin-bottom: 20px;
		overflow: hidden; /* DON'T CHANGE THIS LINE */
	}";

// set layout according to "disp" argument :
$disp = getVar("disp");

if ($disp == "btop" || $disp == "bdown") {
	echo '
	/* global styles for the chessboard container */
	.board_panel {
		width: '; echo ($boardSize + 2*$padding); echo 'px;
		margin: 0 auto;
		text-align: center;
		/* you can EDIT properties FROM HERE ONLY */
		padding-top: '.$padding.'px;
		padding-bottom: '.$padding.'px;
		margin-bottom: '.$padding.'px;
		font-size: 13pt;
		line-height: 12pt;
	}
	/* global styles for the game moves container */
	.header_moves {
		/* you can EDIT everything here */
		background-color: #E0E0E0;
		padding-left: '.$padding.'px;
		padding-right: '.$padding.'px;
		padding-bottom: 1px;
		font-size: 12.5pt;
		line-height: 16pt;
	}
';
}

else if ($disp == "bleft") {
	echo '
	/* global styles for the chessboard container */
	.board_panel {
		float: left;
		width: '; echo ($boardSize + 2*$padding); echo 'px;
		text-align: center;
		padding-top: '.$padding.'px;
		padding-bottom: '.$padding.'px;
		/* you can EDIT all properties except margin FROM HERE ONLY */
		font-size: 13pt;
		line-height: 12pt;
	}
	/* global styles for the game moves container */
	.header_moves {
		margin-left: '; echo ($boardSize + 2*$padding); echo 'px;
		/* you can EDIT properties FROM HERE ONLY */
		background-color: #E0E0E0;
		padding-left: '.$padding.'px;
		padding-right: '.$padding.'px;
		padding-bottom: 1px;
		font-size: 12.5pt;
		line-height: 16pt;
	}
';
}

else if ($disp == "bright") {
	echo '
	/* global styles for the chessboard container */
	.board_panel {
		float: right;
		width: '; echo ($boardSize + 2*$padding); echo 'px;
		text-align: center;
		padding-top: '.$padding.'px;
		padding-bottom: '.$padding.'px;
		/* you can EDIT all properties except margin FROM HERE ONLY */
		font-size: 13pt;
		line-height: 12pt;
	}
	/* global styles for the game moves container */
	.header_moves {
		margin-right: '; echo ($boardSize + 2*$padding); echo 'px;
		/* you can EDIT properties FROM HERE ONLY */
		background-color: #E0E0E0;
		padding-left: '.$padding.'px;
		padding-right: '.$padding.'px;
		padding-bottom: 1px;
		font-size: 12.5pt;
		line-height: 16pt;
	}
';
}

// then, javascript scripts :
echo '</style>

<script type="text/javascript">
	// initialize useful javascript variables :

	var lmovecol = "#990000"; // color of outlined move
	var movescol = 0; // color of other moves (this var is filled by looking at the HTML)
	var doc = document; // main document (less chars to type..)

	// paths to pieces files and empty square :
	var bp = "'.$ep["bp"].'";
	var br = "'.$ep["br"].'";
	var bn = "'.$ep["bn"].'";
	var bb = "'.$ep["bb"].'";
	var bq = "'.$ep["bq"].'";
	var bk = "'.$ep["bk"].'";
	var wp = "'.$ep["wp"].'";
	var wr = "'.$ep["wr"].'";
	var wn = "'.$ep["wn"].'";
	var wb = "'.$ep["wb"].'";
	var wq = "'.$ep["wq"].'";
	var wk = "'.$ep["wk"].'";
	var empty = "'.$ep["empty"].'"; // empty square

	var ctrp = "'.$ep["ctrpan"].'"; // control panel with stepvar inactive
	var ctrpv = "'.$ep["ctrpanv"].'"; // control panel with stepvar active
	var autotime = '.$allVars["autotime_"].'; // time between each move in autoplay
	var autoply = 0; // initially, autoplay is off
	var curgame = -1; // index of game currently played

	var kibitz = 0; // telling if user has begun moving pieces [only if kibitz mode]';
if ($allVars["kibitz_"]) {
	echo '
	var bgsqcol = "#6600FF"; // color of highlighted initial square when kibitzing
	var sqsize = '.$squareSize.'; // square size
	// initial square (pixel coords.) in case of kibitzing :
	var initSquare = [ ';
	for ($i=0; $i < $totGames; $i++) {
		echo '[-1,-1]';
		if ($i < $totGames - 1) echo ', ';
	}
	echo ' ];
	// destination square (pixel coords.) in case of kibitzing :
	var destSquare = [ ';
	for ($i=0; $i < $totGames; $i++) {
		echo '[-1,-1]';
		if ($i < $totGames - 1) echo ', ';
	}
	echo ' ];
	// chessboards upper left corners (still pixel coords.) :
	var chcoords = [ ';
	for ($i=0; $i < $totGames; $i++) {
		echo '[-1,-1]';
		if ($i < $totGames - 1) echo ', ';
	}
	echo ' ];
	// and "stacks" to remember kibitzed moves :
	var stackMoves = [ ';
	for ($i=0; $i < $totGames; $i++) {
		echo '[]';
		if ($i < $totGames - 1) echo ', ';
	}
	echo ' ];';
}

echo '

	// strings representing games :
	var games = [';
for ($i = 0; $i < $totGames; $i++) {
	echo '
"'.$sg["asciiMoves"][$i].'"';
	if ($i < $totGames - 1) echo ',';
}
echo '
	];
	var curinds = [';
for ($i = 0; $i < $totGames; $i++) {
	echo '-1';
	if ($i < $totGames - 1) echo ',';
}
echo ']; // current indices for moving inside games
	var imginds = [';
for ($i = 0; $i < $totGames; $i++) {
	echo '-1';
	if ($i < $totGames - 1) echo ',';
}
echo ']; // indices of first chessboards images in document.images[]
	var flipped = [';
for ($i = 0; $i < $totGames; $i++) {
	echo '0';
	if ($i < $totGames - 1) echo ',';
}
echo ']; // tells if the board is flipped
	var stepvar = [';
for ($i = 0; $i < $totGames; $i++) {
	echo '0';
	if ($i < $totGames - 1) echo ',';
}
echo ']; // step into variations, off by default
	// internal boards to move without printings :
	var boards = [';
for ($i = 0; $i < $totGames; $i++) {
	echo '
[-2,-3,-4,-5,-6,-4,-3,-2,-1,-1,-1,-1,-1,-1,-1,-1,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
1,1,1,1,1,1,1,1,2,3,4,5,6,4,3,2]';
	if ($i < $totGames - 1) echo ',';
}
echo '
	];
</script>

<!-- functions to move pieces forward, according to the games -->
<script type="text/javascript" src="./sources/src_JS/playon.js"> </script>

<!-- functions to move back pieces, according to the games -->
<script type="text/javascript" src="./sources/src_JS/takeback.js"> </script>

<!-- several other functions, like GotoMove() ... -->
<script type="text/javascript" src="./sources/src_JS/dynamics.js"> </script>
';

// include the file for kibitzing only if asked by user :
if ($allVars["kibitz_"]) {
	echo '
<!-- functions to move pieces freely ("kibitz" mode) -->
<script type="text/javascript" src="./sources/src_JS/kibitz.js"> </script>
';
}

// function to be run at the beginning (if JS activated) :
	echo '
<script type="text/javascript">
	function JS_at_start() {';

if ($allVars["jswarn_"]) {
	echo '
		// if we are here, it means that JS is enabled ; so hide warning :
		doc.getElementById("jsdisabled").style.display = "none";
';
}

	echo '
		// initialize movescol with the color of first move of first game
		movescol = doc.getElementById("m0_0").style.color;

		// assign the listener for keyboard events :
		doc.onkeydown = HandleKeys;';

if ($disp == "bleft" || $disp == "bright") {
	echo '

		// center the chessboards vertically :
		var tabBP = doc.getElementsByClassName("board_panel");
		var tabHM = doc.getElementsByClassName("header_moves");
		for (i=0; i<tabBP.length; i++) {
			var nheight = Math.max('; echo ($boardSize + 2*$padding + 2*$ep["hcpan"] + 10); echo ', tabHM[i].offsetHeight);
			var margUpDown = ( nheight - '; echo ($boardSize + 2*$padding + 2*$ep["hcpan"] + 10); echo ' ) / 2;
			tabBP[i].style.marginTop = margUpDown+"px";
			tabBP[i].style.marginBottom = margUpDown+"px";
			tabHM[i].style.height = nheight+"px";
		}';
}

// include the variables for kibitzing only if asked by user :
if ($allVars["kibitz_"]) {
	echo '

		//get a table of chessboards upper-left corners coordinates
		InitCornersCoords();

		// assign the listener for mouse clicks (kibitz)
		doc.onclick = OnMouseClick;';
}

echo '
	}
</script>
';

echo '
</head>


<body onload="JS_at_start()">
';
if ($allVars["jswarn_"]) {
	echo '<div id="jsdisabled">
	WARNING : JavaScript is disabled in your browser ; enable it to move through games.
</div>
';
}

// include functions for printing HTML to file :
include ("viewer_printht.php");


/************************************************
*************************************************/
// will now print main game elements to the standard output :


// print crosstable first, if asked :
if ($allVars["crosstab_"]) {
	echo '<div id="ctdiv">
';
	printCT($sg["headers"]);
	echo '
</div>
';
}

$curGame = 0;
while ($curGame < $totGames) {

	echo '
<!--
==================================================
BEGINNING OF '; echo ($curGame + 1); echo '-th game
==================================================
-->

<div id="gm'.$curGame.'" class="container" onClick="SetGame('.$curGame.')">

';

	if ($disp != "bdown") {
		// printing chessboard/panel first (standard way)

		/*****
		// write chessboard to the up/or/left AND control panel
		*****/

		echo '<!-- container for chessboard and control panel -->
<div class="board_panel">

';

		printInitialBoard($ep, $curGame);
		printControlPanel($ep, $curGame, $allVars["simpctrp_"]);

		echo '

</div>
';

		/*****
		// write header and game moves
		*****/

		echo '
<!-- container for game informations and moves -->
<div class="header_moves">

';

		printHeader($sg["headers"][$curGame]);
		printGame($curGame, $sg["printableMoves"][$curGame], $allVars["figurs_"], $allVars["nagsfont_"],$allVars["movelks_"],
			      $sg["comments"][$curGame], $allVars["varout_"], $sg["headers"][$curGame]["Result"]);

		echo '

</div>
';
	}

	else {
		// printing chessboard/panel last (emphasize moves the most)

		/*****
		// write header and game moves
		*****/

		echo '<!-- container for game informations and moves -->
<div class="header_moves">

';

		printHeader($sg["headers"][$curGame]);
		printGame($curGame, $sg["printableMoves"][$curGame], $allVars["figurs_"], $allVars["nagsfont_"],$allVars["movelks_"],
                $sg["comments"][$curGame], $allVars["varout_"], $sg["headers"][$curGame]["Result"]);

		echo '

</div>
';

		/*****
		// write chessboard to the right/or/down AND control panel
		*****/

		echo '
<!-- container for chessboard and control panel -->
<div class="board_panel">

';

		printInitialBoard($ep, $curGame);
		printControlPanel($ep, $curGame, $allVars["simpctrp_"]);

		echo '

</div>
';
	}

	echo '
</div>

';

	$curGame++;
}

if ($allVars["pgnlink_"]) {
	echo '
<div id="pgn">Link to <a href="'; echo './sources/'; echo substr($gamesFileName, 3); echo '">PGN games</a>';
	if ($sg["specialSymbs"]) {
		echo '<br> <br>
<span id="pgnwarn">WARNING : some games contain unusual markers, which can be misunderstood by other readers</span>';
	}
	echo '
</div>
';
}

echo '
</body>
</html>
';

?>

