<?php

// main functions for parsing PGN file

// process move string ; divided into at most 3 tokens
function processMove($gameStr) {

    // [optional] FIRST TOKEN : dots, digits, spaces..
    $i = 0;
    while ($gameStr[$i] == '.' || ($gameStr[$i] >= '0' && $gameStr[$i] <= '9') || $gameStr[$i]==' '
           || $gameStr[$i]=="\n" || $gameStr[$i]=="\r" || $gameStr[$i]=="\t") $i++;

    // SECOND TOKEN : the move itself with NO dots at the beginning
    while (! strpos("_ \n\r\t()[]{",$gameStr[$i])) $i++;

    // [optional] THIRD TOKEN : the nag $xx
    while ( strpos("_ \n\r\t",$gameStr[$i]) ) $i++;
    if ($gameStr[$i] == '$') {
		$i++;
        while ($gameStr[$i] >= '0' && $gameStr[$i] <= '9') $i++;
    }

	return $i;
}

// include the nags printings here, because we could need them in comments :
include_once ("nags_arrays.php");

// comments analyser :
function commentsAnalyzer($gameStr, $nagsfont) {

	$specialSymbs = FALSE; // are there special symbol(s) to mark continuation moves, null moves ?
	$comments = array();
	$curTokNum = 0;
	$i = 0;
    while (TRUE) {

        // find the next non space character
        while (strpos("_ \n\r\t",$gameStr[$i])) $i++;

		// end game ?
        if ($gameStr[$i] == '*' || $gameStr[$i] == '0') {
			return array(
					"comms" => $comments,
					"ssymb" => $specialSymbs
					);
		}
        if ($gameStr[$i] == '1' && ($gameStr[$i+1]=='-' || $gameStr[$i+1]=='/')) {
			return array(
					"comms" => $comments,
					"ssymb" => $specialSymbs
					);
		}

        // first digit (or dot or letter a-h, RNBQK, O if black) of a game move :
        if (($gameStr[$i] >= '1' && $gameStr[$i] <= '9') || ($gameStr[$i] >= 'a' && $gameStr[$i] <= 'h')
                || strpos("_RNBQK",$gameStr[$i]) || $gameStr[$i] == 'O' || $gameStr[$i] == '.') {
            $i += processMove( substr($gameStr, $i) );
            $curTokNum++;
        }

		// an isolated nag, like "with the idea of.." :
		else if ($gameStr[$i] == '$') {
			// just loop until end of digits
			$i++;
			while ($gameStr[$i] >= '0' && $gameStr[$i] <= '9') $i++;
			$curTokNum++;
		}

		// mark for continuation move [is NOT a token ! ..but '(' and '[' ARE ! ] :
		else if (($gameStr[$i] == '(' || $gameStr[$i] == '[') && $gameStr[$i+1] == '*') {
			$specialSymbs = TRUE;
			$i += 2;
			$curTokNum++;
		}

		// null move (is NOT a token !) :
		else if ($gameStr[$i] == '<' || $gameStr[$i] == '-') {
			$specialSymbs = TRUE;
			$i += 2;
		}

        else if ($gameStr[$i] == '{') {
            // look through until '}'

            // eliminate spaces and lines jumps :
            $i++;
            while (strpos("_ \n\r\t",$gameStr[$i])) $i++;

            // now find end of comment :
            $j = $i;
            while ($gameStr[$j] != '}') $j++;
			$nextPos = $j + 1; // important : skip '}' to get right $curTokNum !
            // ..and step back while space, tab or endline met :
			$j--;
            while (strpos("_ \n\r\t",$gameStr[$j])) $j--;

			// we got comment :
            $comments[$curTokNum] = substr($gameStr,$i,$j-$i+1);
			$i = $nextPos;
        }

        else { // includes closing parenthesis and brackets skipping
			if ($gameStr[$i]=='[' || $gameStr[$i]==']') $specialSymbs = TRUE;
			$i++;
			$curTokNum++;
		}
    }

    return ""; // unreachable
}

// MAIN LOOP : parse all PGN headers and games moves
// (only 3 optional fields could be parsed : the elo(s), and eco code)
// [should be rewrite in AJAX later..]

function parseAllGames($gamesFileName, $nagsfont) {

	$games = fopen($gamesFileName, "rb");
	$curGame = 0;

	// prepare results arrays :
	$headers = array();
	$printableMoves = array();
	$asciiMoves = array();
	$comments = array();
	$specialSymbs = FALSE; // are there special symbol(s) to mark continuation moves, null moves ?

	// process file line by line :
	while( ! feof($games) ) {

		// find the first header line ; assume then following lines are header too
		do {
			$curLine = fgets($games);
			if ( feof($games) ) break 2;
		} while ($curLine[0] != '[');

		// a new game header is found
		// process each header information until we meet blank line or first move :
		while ($curLine[0] == '[') {
			$firstChar = 1;
			while ($curLine[$firstChar] == ' ' || $curLine[$firstChar] == '\t') $firstChar++;
			$lastChar = $firstChar;
			while (! strpos("_ \t\"",$curLine[$lastChar])) $lastChar++;
			// get field name
			$field_name = substr($curLine, $firstChar, $lastChar - $firstChar);

			// read _something_ :
			while ($curLine[$lastChar] == ' ' || $curLine[$firstChar] == '\t') $lastChar++;
			$lastChar++; // go over the " mark
			$firstChar = $lastChar;
			while ($curLine[$lastChar] != "\"") $lastChar++;
			$field_value = substr($curLine, $firstChar, $lastChar - $firstChar);

			// matching stage :
			$headers[$curGame][$field_name] = $field_value;
			$curLine = fgets($games);
		}

		// now find the first line of game moves :
		while ($curLine[0] == "\n" || $curLine[0] == "\r") $curLine = fgets($games);

		// parse game moves : the most difficult part ! I use an external program written in C for fast processing
		// we get the string "palview like" AND the sequence of moves "textual" to put aside of the board.
		$moves = '';
		while ( ! feof($games) ) {
			$moves .= $curLine.' ';
			$curLine = fgets($games);
			if ($curLine[0] == "\n" || $curLine[0] == "\r") break; // line jump mark end of game			
		}

		// execute compiled C program :
		$gameStrings = exec('../mparser.exe '.escapeshellarg($moves));
		$printableMoves[$curGame] = strtok($gameStrings, '*');
		$asciiMoves[$curGame] = strtok('*');
		$comAndSymb = commentsAnalyzer($moves, $nagsfont);
		$comments[$curGame] = $comAndSymb["comms"];
		$specialSymbs = ($specialSymbs || $comAndSymb["ssymb"]);

		$curGame++;
	}

	fclose($games);
	return ( array(
		"headers" => $headers,
		"printableMoves" => $printableMoves,
		"asciiMoves" => $asciiMoves,
		"comments" => $comments,
		"specialSymbs" => $specialSymbs
	));
}

?>

